/**
 * Copyright © 2010 Nokia Corporation. All rights reserved. Nokia and Nokia Connecting People
 * are registered trademarks of Nokia Corporation. Java and all Java-based marks are trademarks
 * or registered trademarks of Sun Microsystems, Inc. Other product and company names mentioned herein
 * may be trademarks or trade names of their respective owners.
 *
 * Subject to the conditions below, you may, without charge:
 *  - Use, copy, modify and/or merge copies of this software and associated documentation files (the "Software")
 *  - Publish, distribute, sub-license and/or sell new software derived from or incorporating the Software.
 *
 * This license, unmodified, shall be included with all copies or substantial portions of the Software
 * that are distributed in source code form.
 *
 * The Software cannot constitute the primary value of any new software derived from or incorporating the Software.
 * Any person dealing with the Software shall not misrepresent the source of the Software.
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
 * AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;
import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreException;

/**
 * MIDlet for benchmarking different record store access strategies.
 */
public class RmsMidlet extends MIDlet implements CommandListener {
    private Command exit;
    private Command run;
    private Command delete;
    private Form form;

    private long startTime;
    private long stopTime;

    private RmsConnection basicRms = new BasicRms();
    private RmsConnection badRms = new BadRms();
    private RmsConnection cacheRms = new CacheRms();
    private RmsConnection fileRms = new FileRms();

    private final String recordData = "some dummy data for record";

    public RmsMidlet() {
        form = new Form("RMS");
        exit = new Command("Exit", Command.EXIT, 1);
        run = new Command("Run", Command.OK, 1);
        delete = new Command("Delete", Command.OK, 1);
        form.addCommand(exit);
        form.addCommand(run);
        form.addCommand(delete);
        form.setCommandListener(this);
    }

    public void startApp() {
        Display.getDisplay(this).setCurrent(form);
    }

    public void pauseApp() {
    }

    public void destroyApp(boolean unconditional) {
    }

    public void commandAction(Command command, Displayable displayable)
    {
        if (command == exit)
        {
            destroyApp(true);
            notifyDestroyed();
        }
        else if (command == run)
        {
            run();
        }
        else if (command == delete)
        {
            reset();
        }
    }

    private void log(String msg) {
        System.out.println(msg);
        form.append(msg + "\n");
    }

    private void clear() {
        form.deleteAll();
    }

    private void run() {
        init();
        log("Starting test...");
        String simpleResult = test(basicRms);
        log("Basic RecordStore:\n" + simpleResult);
        String badResult = test(badRms);
        log("Bad RecordStore:\n" + badResult);
        String cacheResult = test(cacheRms);
        log("Cache RecordStore:\n" + cacheResult);
        String fileResult = test(fileRms);
        log("File RecordStore:\n" + fileResult);
        log("Test done");
    }

    private void init() {
        clear();
        String[] stores = RecordStore.listRecordStores();
        if (stores == null) {
            log("Populating recordstores...");
            populateStore(basicRms);
            populateStore(badRms);
            populateStore(cacheRms);
            populateStore(fileRms);
            log("Populating recordstores done");
        }
    }

    private void populateStore(RmsConnection rms) {
        rms.open();
        rms.set(recordData.getBytes());
        rms.close();
    }

    private void reset() {
        clear();
        try {
            log("Removing recordstores...");
            String[] stores = RecordStore.listRecordStores();
            for(int i = 0; stores != null && i < stores.length; i++) {
                RecordStore.deleteRecordStore(stores[i]);
            }
            // remove file
            ((FileRms)fileRms).delete();
            log("Removing recordstores done");

        } catch(RecordStoreException rse) {
            log(rse.toString());
        }
    }

    private String test(RmsConnection rms) {
        startTiming();
        rms.open();
        for(int i = 0; i < 100; i++) {
            byte[] data = rms.get();
            String out = new String(data);
            // sanity check - what goes in must come out
            checkEquals(recordData, out);
        }
        rms.close();
        stopTiming();
        return getTime();
    }

    private void checkEquals(String expected, String actual) {
        if (expected.compareTo(actual) != 0) {
            log("data does not match: expected=" + expected + " actual=" + actual);
        }
    }
    private void startTiming() {
        startTime = System.currentTimeMillis();
    }

    private void stopTiming() {
        stopTime = System.currentTimeMillis();
    }

    private String getTime() {
        return (new Long(stopTime - startTime)).toString() + " ms";
    }
}
